#!/usr/bin/perl -w

# TUCOPY Tucows mirroring software
# Copyright (C) 2001 Tucows Inc.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. 
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details. 
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


#
# vi:set tabstop=4:
# $Id: tucopy.pl,v 1.11 2001/04/12 17:46:36 bjovanovic Exp $
#
# Authors:
#       Bojan Jovanovic <bjovanovic@tucows.com>
#       Leonid Igolnik <lim@tucows.com>
#


use strict;

use Tucows::Mirror::Config;
use Tucows::Mirror::Log;
use Tucows::Mirror::Client::Client_CL_Params;
use Tucows::Mirror::Client::Package;
use vars qw( $VERSION );

$VERSION = '1.14';

# Getting the command line parameters and displaying help/usage if requested
my $params = Tucows::Mirror::Client::Client_CL_Params->get_client_parameters(\@ARGV);

# $params will be undef if command line parsing was not successful
exit(0) unless (defined $params);

set_verbose(1) if $params->v;

# set $ENV{MirrorConfigFile} if -c supplied
$ENV{MirrorConfigFile} = $params->c()
	if (defined $params->c());

# config file name picked up from the enviroment variable $ENV{MirrorConfigFile}
my $config = Tucows::Mirror::Config->new();

die "Unable to open config file"
	unless (defined $config);

# Load all the available/supplied packages in the list of packages
my $packages = Tucows::Mirror::Client::Package->get_packages( $params->list(),
					$config );

# process packages
foreach my $package (@$packages) {
	$package->process($config, $params) || logit('error', 2,
				'E1 Proccessing of %s package failed', $package->package_name);
}

exit;
