#!/usr/bin/perl

# TUCOPY Tucows mirroring software
# Copyright (C) 2001 Tucows Inc.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. 
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details. 
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


#
# vi:set tabstop=4:
# $Id: tustat.pl,v 1.2 2001/04/10 20:14:10 bjovanovic Exp $
#
# Authors:
#       Leonid Igolnik <lim@tucows.com>
#       Bojan Jovanovic <bjovanovic@tucows.com>
#

use strict;
use Data::Dumper;
my %sessions;

while (<>) {
	my ($time, $sessid, $package, $full_size, $fetched_size) =
		(/^\[.+\]\s(\d+)\s(\d+)\s(.+)\s".+"\s(\d+)\s(\d+)\s\d+$/)
		or next;
	$sessions{$sessid} = {} unless defined $sessions{$sessid};
	$sessions{$sessid}->{packages} = {}
		unless defined $sessions{$sessid}->{packages};
	$sessions{$sessid}->{sess_start} = $time
		unless exists $sessions{$sessid}->{sess_start};
	$sessions{$sessid}->{sess_end} = $time;
	$sessions{$sessid}->{packages}->{$package}->{num_files}++;
	$sessions{$sessid}->{packages}->{$package}->{content_size} += $full_size;
	$sessions{$sessid}->{packages}->{$package}->{fetched_size} += $fetched_size;
}

#print Dumper(\%sessions);

my $format = "| %8s | %-10s | %10d | %12d | %12d | %8.2f %%|\n";
print ' ', '-' x 78, "\n";
printf "| %-8s | %-10s | %-10s | %-12s | %-12s | %-9s |\n",
		'Session', 'Package', 'Num. files', 'Content size',
		'Fetched size', 'Savings';
print ' ', '-' x 78, "\n";
my %totals;
foreach my $sessid (sort {$sessions{$a}->{sess_start} <=> $sessions{$b}->{sess_start}} (keys %sessions)) {
	foreach my $package (keys %{$sessions{$sessid}->{packages}}) {
		printf $format,
				$sessid,	
				uc($package),
				$sessions{$sessid}->{packages}->{$package}->{num_files},
				$sessions{$sessid}->{packages}->{$package}->{content_size} /
					1024,
				$sessions{$sessid}->{packages}->{$package}->{fetched_size} /
					1024,
				100 -
				($sessions{$sessid}->{packages}->{$package}->{fetched_size} /
				$sessions{$sessid}->{packages}->{$package}->{content_size}) * 
				100;
		$totals{packages} = {} unless exists $totals{packages};
		$totals{packages}->{$package} = {}
				unless exists $totals{packages}->{$package};
		$totals{packages}->{$package}->{num_files} +=
				$sessions{$sessid}->{packages}->{$package}->{num_files};
		$totals{packages}->{$package}->{content_size} +=
				$sessions{$sessid}->{packages}->{$package}->{content_size};
		$totals{packages}->{$package}->{fetched_size} += 
				$sessions{$sessid}->{packages}->{$package}->{fetched_size};
		$totals{num_files} +=
				$sessions{$sessid}->{packages}->{$package}->{num_files};
		$totals{content_size} +=
				$sessions{$sessid}->{packages}->{$package}->{content_size};
		$totals{fetched_size} += 
				$sessions{$sessid}->{packages}->{$package}->{fetched_size};
	}
} 
print ' ', '-' x 78, "\n";
printf "| %-8s | %-10s | %-10s | %-12s | %-12s | %-9s |\n",
	'Totals';
foreach my $package (keys %{$totals{packages}}) {
	printf $format,
				'',
				uc($package),
				$totals{packages}->{$package}->{num_files},
				$totals{packages}->{$package}->{content_size} /
					1024,
				$totals{packages}->{$package}->{fetched_size} /
					1024,
				100 -
				($totals{packages}->{$package}->{fetched_size} /
				$totals{packages}->{$package}->{content_size}) * 
				100;

}
print ' ', '-' x 78, "\n";
printf $format,
		'All',
		'',
		$totals{num_files},
		$totals{content_size} / 1024,
		$totals{fetched_size} / 1024,
		100 - ($totals{fetched_size} / $totals{content_size}) * 100;
print ' ', '-' x 78, "\n";
