#!/usr/bin/perl -w

# TUCOPY Tucows mirroring software
# Copyright (C) 2001 Tucows Inc.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. 
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details. 
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


#
# vi:set tabstop=4:
# $Id: install.pl,v 1.24 2001/04/27 20:10:04 joey Exp $
#
# Authors:
#       Leonid Igolnik <lim@tucows.com>
#       Bojan Jovanovic <bjovanovic@tucows.com>
# 


use File::Path;
use IO::File;

use strict;

my $files = {
	bin => {

	},
	config => {

	},
	lib => {

	},	
	logs => {

	},
	doc => {
	}
};

print <<EOM;

Welcome to the Tucows Content Distribution client installation

Please read the INSTALL and README.cpan files before proceeding

The installer will now prompt you with a few questions to help
set up the basic configuration of the script. 

EOM

# Please run install.pl as root as you will need to install perl modules
do {
	print "You have to run install.pl as root! (uid == 0)\n";
	exit(1);
} if ($< != 0);

my $client_uid = 0;
my $effective_username;
do {
	print "You have to enter valid, existing username\n" 
		if (not defined $client_uid);
	$effective_username = ask('Username of the user client will run as','');
	$client_uid = getpwnam( $effective_username );
} while (not $client_uid);


user_permissions($client_uid);

my $perl_location = '';
$perl_location = '/usr/bin/perl' if -x '/usr/bin/perl';
do {
	$perl_location = ask('perl location', $perl_location);
} while (!-x $perl_location || !-f $perl_location);

my $mirror_root = '/mirror';
$mirror_root = ask("Local content root (must be writeable by $effective_username)", 
	$mirror_root);

my $use_compression = 'Y';
$use_compression = ask_YN('Do you want to use compression', $use_compression);

my $has_compress_zlib;
my $has_gzip;
my $gzip_location;
my $uncompress_mech;
if ($use_compression eq 'Y') {
	my $install_compress_zlib = 'Y';
	do {
		print "Checking for Compress::Zlib: ";
		eval ('require Compress::Zlib');
		if ($@) {
			print "not found.\n";
			$has_compress_zlib = 0;
			$install_compress_zlib =
					ask_YN('Do you want to install Compress::Zlib now',
						$install_compress_zlib);
			if ($install_compress_zlib eq 'Y') {
				root_permissions();
				print "Launching CPAN\n";
				eval ('	
					require CPAN;
					my $obj = CPAN::Shell->expand("Module", "Compress::Zlib");
					$obj->install;
				');
				if ($@) {
					print "\n\nCompress::Zlib installation failed\n".
						"Reason:$@\n\n";
				}
				user_permissions($client_uid);
			}
		} else {
			print "found.\n";
			$has_compress_zlib = 1;
			$uncompress_mech = 'Zlib';
		}
	} while ($install_compress_zlib eq 'Y' && $has_compress_zlib == 0);

	my $disable_compression = '';	
	if ($has_compress_zlib == 0) {
		$gzip_location = '/usr/bin/gzip' if -x '/usr/bin/gzip';
		my ($done);
		do {
		   $gzip_location = ask('gzip location', $gzip_location);
			if (!-x $gzip_location || !-f $gzip_location) {
				print "$gzip_location has no execution permission\n";
				$done = 0;
				$disable_compression  = ask_YN('disable compression', 'N');
				$done = 1 if $disable_compression eq 'Y';
			} else {
				$done = 1;
				$has_gzip = 1;
				$uncompress_mech = 'Gzip';
			}
		} while (!$done);
	}

	if ($disable_compression eq 'Y') {
		print "Compression disabled\n";
		$use_compression = 'N';
	}
}

my $delivery_mech = 'LWP';
my ($wget_location, $lynx_location, $has_lwp);

$delivery_mech = ask_multiple('Select delivery mechanism', 'LWP', 'LWP',
		'Wget', 'Lynx');

my $check_delivery_mech = {
	Wget => sub {
		print "Wget configuration:\n";
		$wget_location = `which wget`;
		chomp $wget_location;
		$wget_location = ask('Wget location', $wget_location);
	},
	Lynx => sub {
		print "Lynx configuration:\n";
		$lynx_location = `which lynx`;
		chomp $lynx_location;
		$lynx_location = ask('Lynx location', $lynx_location);
	},
	LWP => sub {
		my $install_lwp = 'Y';
		$has_lwp = 0;
		do {
			print "Checking for LWP:";
			eval('require LWP::UserAgent');
			if ($@) {
				print "not found\n";
				$install_lwp = ask_YN('Do you want to install LWP now',
						'Y');
				if ($install_lwp eq 'Y') {
					root_permissions();
					print "Launching CPAN\n";
					eval ('
						require CPAN;
						my $obj = CPAN::Shell->expand("Module", "LWP");
						$obj->install;
					');
					if ($@) {
						print "\n\nLWP installation failed\n".
						"Reason: $@\n\n";
					}
					user_permissions($client_uid);
				} else {
					$has_lwp = 0;
				}
			} else {
				print "found\n";
				$has_lwp = 1;
			}
		} while (not $has_lwp and $install_lwp eq 'Y');
	}
};

&{$check_delivery_mech->{$delivery_mech}};



my $target = '/usr/local/tucopy';
$target = ask("Client root (must be writeable by $effective_username)", $target);

if (!-d $target) {
	my $make_dir = ask_YN('Client root dir does not exist, create now', 'Y');
	if ($make_dir eq 'Y') {
		mkpath($target, 0, 0755) || die "Can't create $make_dir: $!";
	}
}
my $tmp_dir = '/tmp';
$tmp_dir = ask('Temp dir', $tmp_dir);

if (!-d $tmp_dir) {
	my $make_dir = ask_YN('Temp dir does not exist, create now', 'Y');
	if ($make_dir eq 'Y') {
		mkpath($tmp_dir, 0, 0755) || die "Can't create $make_dir: $!";
	}
}

my $copy_command = `which cp`;
chomp $copy_command;
$copy_command = ask('cp location', $copy_command) unless $copy_command;

my $user_name = '';
$user_name = ask('Enter your affiliate username', $user_name);

my $user_pass = '';
$user_pass = ask('Enter your affiliate password', $user_pass);

my $max_speed = 0;
$max_speed = ask('Enter max download speed, 0 to igonore', $max_speed);

my $params = {
	perl_location		=> $perl_location,
	use_compression		=> (lc $use_compression eq 'y') ? 1 : 0,
	uncompress_mech		=> $uncompress_mech,
	gzip_location		=> $gzip_location,
	has_compress_zlib	=> $has_compress_zlib,
	delivery_mech		=> $delivery_mech,
	wget_location		=> $wget_location,
	lynx_location		=> $lynx_location,
	has_lwp				=> $has_lwp,
	target				=> $target,
	tmp_dir				=> $tmp_dir,
	copy_command		=> $copy_command,
	mirror_root			=> $mirror_root,
	user_name			=> $user_name,
	user_pass			=> $user_pass,
	max_speed			=> $max_speed,
};

print "\nClient will be installed with following configuration:\n";
foreach my $key (keys %$params) {
	next unless defined $params->{$key};
	print "$key => $params->{$key}\n";
}
print "\nPress [ENTER] to continue\n";
<>;

install_files($files, $params);
configure($params);
write_shell_script($params);

sub write_shell_script {
	my ($params) = @_;
	
	my $shell_file_name = $params->{target} . '/bin/tucopy.sh';
	my $fh = IO::File->new($shell_file_name, 'w') ||
				die "Unable to open $shell_file_name for writing: $!\n";
	print $fh <<EOSH;
#!/bin/sh
MirrorConfigFile=$params->{target}/config/tucopy.cfg
export MirrorConfigFile
$params->{perl_location} -I $params->{target}/lib $params->{target}/bin/tucopy.pl "\$\@"
EOSH

	$fh->close || die "Unable to close $shell_file_name: $!\n";
	chmod 0755, $shell_file_name;
}

sub configure {
	my ($params) = @_;

	print "Writing configuration files :";
	my @conf_files = 'config/tucopy.cfg';
	my $client_uid = $>;

	# become root - spawning child as non-priviledged effective user id fails
	root_permissions($client_uid);
	push @conf_files, <config/packages/*.pkg>;
	user_permissions($client_uid);	

	foreach my $file_name (@conf_files) {
		open(SRC, $file_name) || die "Can't open $file_name: $!\n";
		my @src = <SRC>;
		close(SRC);
		my $final_config = $params->{target} . '/' . $file_name;
		open(TGT, '>' . $final_config) ||
			die "Unable to open $final_config: $!\n";
		foreach my $line (@src) {
			$line =~ s/TARGET/$params->{target}/og;		
			$line =~ s/MIRROR_ROOT/$params->{mirror_root}/go;
			$line =~ s/DELIVERY_MECH/$params->{delivery_mech}/og;
			$line =~ s/WGET_LOCATION/$params->{wget_location}/og
						if $params->{wget_location};
			$line =~ s/LYNX_LOCATION/$params->{lynx_location}/og
						if $params->{lynx_location};
			$line =~ s/TMP_DIR/$params->{tmp_dir}/og;
			$line =~ s/UNCOMPRESS_MECH/$params->{uncompress_mech}/og 
						if $params->{uncompress_mech};
			$line =~ s/GZIP_LOCATION/$params->{gzip_location}/og
						if $params->{gzip_location};
			$line =~ s/PACKAGE_USR/$params->{user_name}/g;
			$line =~ s/PACKAGE_PASS/$params->{user_pass}/g;
			$line =~ s/MAX_SPEED/$params->{max_speed}/g;
			$line =~ s/USE_COMPRESSION/$params->{use_compression}/g;
			print TGT $line;
		}
		close(TGT) || die "Unable to close $final_config: $!\n";
	}
	print "done\n";
}

sub install_files {
	my ($files, $params) = @_;

	foreach my $file (keys %$files) {
		if (-d $file) {
			copy_dir($file, $params);
		};
	}
}

sub copy_dir {
	my ($dir, $params) = @_;

	mkpath($params->{target} . '/' . $dir);

	opendir(DIR, $dir) || die "Can't open dir $dir: $!\n";
	my @files = readdir(DIR);
	close(DIR);

	foreach my $file (@files) {
		next if $file =~ /^\./;
		if (-d $dir . '/' . $file) {
			copy_dir($dir . '/' . $file, $params);
		} else {
			print "Copying $dir/$file to $params->{target}/$dir/$file\n";
			system $params->{copy_command}, $dir . '/' . $file,
					$params->{target} . '/' . $dir . '/' . $file;
		}
	}
	
}

sub ask_multiple {
	my ($ask, $default, @choices) = @_;
	my $choices = join('/', @choices);
	my %choices = map {$_ => 1} @choices;
	my $inp;
	do {
		$inp = ask($ask . ' ' . $choices, $default);
		chomp $inp;
	} while (not defined $choices{$inp});
	return $inp;
}

sub ask_YN {
	my ($ask, $default) = @_;
	my $inp;
	do {
		$inp = ask($ask . ' Y/N', $default);
		$inp = uc($inp);
	} while ($inp ne 'Y' && $inp ne 'N');
	return $inp;
}

sub ask {
	my ($ask, $default) = @_;
	print "$ask [$default]? ";
	my $inp = <>;
	if ($inp =~ /\S/) {
		chomp $inp;
		return $inp;
	} else {
		return $default;
	}
}

sub root_permissions {
	$> = 0;
	die "Unable to regain root permissions!" unless ($> == 0);
}

sub user_permissions {
	my $uid = shift;
	$> = $uid;
	die "Unable to get user permissions!" unless ($> == $uid);
}
