# TUCOPY Tucows mirroring software
# Copyright (C) 2001 Tucows Inc.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. 
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details. 
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


#
# vi:set tabstop=4:
# $Id: CL_Params.pm,v 1.6 2001/03/12 15:58:49 bjovanovic Exp $
#
# Authors:
#       Bojan Jovanovic <bjovanovic@tucows.com>
#       Leonid Igolnik <lim@tucows.com>
#

package Tucows::Mirror::CL_Params;

use vars qw( $AUTOLOAD );
use strict;

# Simple generic command line options parser
# assumes the following format:
# [[-switchname [switchparams]] .... ] [elem1 ...]
# 
# list of elems can be retreived with ->list() method,
# value(s) for each switch can be retreived with ->$switchname()
#
#	Sample parameter description:
#	
#	$param_desc = 
#			{
#			force => 
#				{
#				type => 'switch',
#				default => 0,
#				},
#			sync => 
#				{
#				type => 'switch',
#				default => 0,
#				},
#			c =>
#				{
#				type => 'value',
#				default => undef,
#				}
#			};
		

sub switch {
	my $self = shift;
	my ($param_name, $ARGV) = @_;

	$self->{_options}->{$param_name} = 1;
	return 1;
}

sub value {
	my $self = shift;
	my ($param_name, $ARGV) = @_;

	my $value = shift @$ARGV;

	$self->{_options}->{$param_name} = $value;
	return 1;
}


# Constructor function

sub parse_parameters {
	my $pkg = shift;
	my ($ARGV, $param_desc) = @_;

	my @ARGV = @$ARGV;

	my $self = {
		_options =>{},
		_list => [],
		_param_desc => $param_desc,
	};

	bless $self, $pkg;

	my @list = ();

	while (@ARGV) {
		my $param_name = shift(@ARGV);

		if ($param_name =~ s/^\-//) {
			if (defined $self->{_param_desc}->{$param_name}) {
				my $method_name = $self->{_param_desc}->{$param_name}->{type};
				if ($self->can($method_name)) {
					$self->$method_name( $param_name, \@ARGV );
				} else {
					die "Unknown input option type $method_name\n";
				}
			} else {
				warn "Unknown parameter $param_name\n";
				return undef;
			}
		} else {
			# ran out of parameters -> list
			@list = ($param_name, @ARGV);
			@ARGV = ();
		}
	}
	$self->{_list} = \@list;

	return $self;
}

sub list {
	my $self = shift;
	return $self->{_list};
}

sub DESTROY {
	return 1;
}

sub AUTOLOAD {
	my $self = shift;
	my $type = ref($self)
		or die "$self is not an object... In attempted call to $AUTOLOAD";

	my $name = $AUTOLOAD;

	$name =~ s/.*://;   # strip fully-qualified portion

	if (exists($self->{_options}->{$name}) ) {
		return $self->{_options}->{$name};
	} elsif ( defined $self->{_param_desc}->{$name}) {
		return $self->{_param_desc}->{$name}->{default};
	} else {
		warn "Unknown parameter $name requested\n";
		return undef;
	}
}

1;
