# TUCOPY Tucows mirroring software
# Copyright (C) 2001 Tucows Inc.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. 
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details. 
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


#
# vi:set tabstop=4:
# $Id: Client_CL_Params.pm,v 1.8 2001/10/23 20:01:23 lim Exp $
#
# Authors:
#       Bojan Jovanovic <bjovanovic@tucows.com>
#       Leonid Igolnik <lim@tucows.com>
#

package Tucows::Mirror::Client::Client_CL_Params;

use Tucows::Mirror::CL_Params;

@ISA = qw ( Tucows::Mirror::CL_Params );

use strict;

# Tucows::Mirror::Client::Client_CL_Params is strictly custom module used 
# in Tucows mirror script

# $params_desc describes the list of the command line parameters 

my $params_desc = {
			force => {
				type => 'switch',
				default => 0,
			},
			sync => {
				type => 'switch',
				default => 0,
			},
			ignoresize => {
				type => 'switch',
				default => 0,
			},
			c => {
				type => 'value',
				default => undef,
			},
			V => {
				type => 'switch',
				default => 0,
			},
			v => {
				type => 'switch',
				default => 0,
			},
			h => {
				type => 'switch',
				default => 0,
			},
			t => {
				type => 'switch',
				default => 0,
			},
			'-help' => {
				type => 'switch',
				default => 0,
			},
		};


sub get_client_parameters {
	my $pkg = shift;
	my $ARGV = shift;
	
	my $self = $pkg->SUPER::parse_parameters($ARGV, $params_desc);

	# needed to do $params->'-help'() (this syntax not supported by perl)
	my $help_name = '-help';

	# if parameter parsing failed, or -h was set, display usage..
	if (not defined $self or $self->h()) {
		# usage string
		print "Usage (use --help for full help): \n".
			"mirror.pl [-h] [--help] [-t] [-v] [-V] [-c name]".
			" [-sync] [-force] [-ignoresize] [package ..]\n";
		$self = undef;
	} elsif ($self->V()) {
		print "Tucows mirror client Version: ".$main::VERSION."\n";
		$self = undef;
	} elsif ($self->$help_name()) {
		print << 'EOHELP';
Tucopy Options

-t      Test mode, do not update the mirror, just show what needs to be updated.
-v      Verbose mode, fully display what the tucopy script is doing
-V      Print the version number
-c      Allows the user to specify an alternative config file
-sync   Sync's the timestamps with the server
-force  Forces a full download/update of all files 
-ignoresize Ignore file size mismatches

EOHELP
		$self = undef;
	}

	return $self;
}

1;
