# TUCOPY Tucows mirroring software
# Copyright (C) 2001 Tucows Inc.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. 
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details. 
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


#
# vi:set tabstop=4:
# $Id: Error_codes.pm,v 1.13 2001/04/12 18:50:52 bjovanovic Exp $
#
# Authors:
#       Bojan Jovanovic <bjovanovic@tucows.com>
#       Leonid Igolnik <lim@tucows.com>
#       Alex Norbert <anorbert@tucows.com>
#

package Tucows::Mirror::Client::Fetch::Error_codes;

require Exporter;
@ISA = qw(Exporter);

@EXPORT_OK = qw( ok_code get_error_type ); 

use strict;

my $OK_CODE = 200;

sub ok_code { $OK_CODE; }

my $_error_codes = {
	200 => {
		message => 'OK',
		success => 1, # File was succesfully fetched
		retry => 0, 
	},
	404 => {
		message => 'Not Found',
		success => 0, # Not a success code
		retry => 0, # There is no point in retrying - file is not there
	},
	401 => {
		message => 'Authorization required',
		success => 0, # Not a success code
		retry => 0, # there is no point in retrying - authorization is missing
	},

	log_open_error => {
		message => 'Error opening log file $0',
		success => 0,
		retry => 0,
	},

	log_parse_error => {
		message => 'Unknown format or message in $0 log file: $1; $2',
		success => 0,
		retry => 1,
	},

	file_size_mismatch => {
		message => 'File size mismatch after fetch $0 (local: $1, remote: $2). Retrying',
		success => 0,
		retry => 1,
	},

	other => {
		message => 'Unknown error or return code occured in fetch driver: $0',
		success => 0, # Not a success code
		retry => 1, # we assume that for all other messages we should retry
	},	

	exec_failed => {
		message => 'Exec on $0 failed',
		success => 0,
		retry => 1,
	},

	openw_failed => {
		message => 'Open (for writing) of $0 failed',
		success => 0,
		retry => 1,
	},

	fork_failed => {
		message => 'fork() in $0 failed',
		success => 0,
		retry => 0,
	},

	wait_on_pid => {
		message => 'Wait on child process (PID $0) failed',
		success => 0,
		retry => 0, # don't create zombies - something is wrong
	},
};

sub get_error_type {
	my $code = shift;

	my $error_type;

	if (defined $_error_codes->{$code}) {
		$error_type = {%{$_error_codes->{ $code }}};
	} else {
		$error_type = {%{$_error_codes->{ other }}};
	}

	$error_type->{message} = _substitute_values( $error_type->{message}, @_ );
	return $error_type;
}

sub _substitute_values {
	my $message = shift;

	for (my $param_nr = 0; $param_nr <= $#_; $param_nr++) {
		$message =~ s/\$(\d+)/(defined($_[$1])?$_[$1]:'')/ge;
	}

	return $message;
}

1;
