# TUCOPY Tucows mirroring software
# Copyright (C) 2001 Tucows Inc.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. 
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details. 
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


#
# vi:set tabstop=4:
# $Id: LWP.pm,v 1.9 2001/03/12 15:58:51 bjovanovic Exp $
#
#
# Authors:
#       Bojan Jovanovic <bjovanovic@tucows.com>
#       Leonid Igolnik <lim@tucows.com>
#       Alex Norbert <anorbert@tucows.com>
#
package Tucows::Mirror::Client::Fetch::LWP;

use HTTP::Request;
use HTTP::Headers;
use LWP::UserAgent;

use Tucows::Mirror::Client::Fetch::Error_codes qw( ok_code get_error_type );

use strict;

my $_driver = 'LWP';

sub fetch {
	my $params = shift;
	
	my $status;

	# Create a user agent object
	my $ua = LWP::UserAgent->new();

	# Create a new headers object
	my $header = HTTP::Headers->new();
	
	# Add authorization username and password to the headers
	$header->authorization_basic( $params->{ User }, $params->{ Pass } );

	# Create a request
	my $req = HTTP::Request->new(GET => $params->{URL}, $header);

	# Pass request to the user agent and get a response back
	# and specify output file location
	my $res = $ua->request($req, $params->{Target});

	# Check the outcome of the response
	if ($res->is_success) {
		$status = get_error_type( ok_code() );
	} else {

		# parse the response
		$res->status_line =~ /^\s*(\d+)/;

		# if parsing failed, $1 will be undefined, and it will trigger 
		# unknown error in get_error_type, and display the entire status line
		$status = get_error_type( $1, $res->status_line() );
	}

	return $status;


}


1;
