# TUCOPY Tucows mirroring software
# Copyright (C) 2001 Tucows Inc.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. 
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details. 
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


#
# vi:set tabstop=4:
# $Id: Lynx.pm,v 1.16 2001/03/12 15:58:51 bjovanovic Exp $
#
# 
#
# Authors:
#	Bojan Jovanovic <bjovanovic@tucows.com>
#	Leonid Igolnik <lim@tucows.com>
#	Alex Norbert <anorbert@tucows.com>


package Tucows::Mirror::Client::Fetch::Lynx;

use strict;
use Tucows::Mirror::Config;
use Tucows::Mirror::Client::Fetch::Error_codes qw( ok_code get_error_type );

# Has init been called?
my $_init = 0;

# following variables will be initialized in _init
my $_config;
my $_lynx_location;
my $_tmp_dir;
my $_lynx_error_log;

# constants

# driver name
my $_driver = 'Lynx';

# Error codes returned from the fork
my $_exec_failed_code = 201; #Should not colide with any of the lynx error codes
my $_exec_failed_open = 202; #Should not colide with any of the lynx error codes

# Null device
my $_null = '/dev/null'; 


# _init function - called only once
sub _init {
	$_config = Tucows::Mirror::Config->new();
	$_lynx_location = $_config->delivery_lynx_location;
	$_tmp_dir = $_config->tmp_dir();
	$_lynx_error_log = "$_tmp_dir/.lynx_log";
	$_init = 1;
	return 1;
}

sub fetch {
	my($opts) = shift;

	_init() 
		unless ($_init);

	my $auth;
	if ($opts->{User}) { $auth = "-auth=$opts->{User}:$opts->{Pass}"; }

	my $pid = fork;

	return get_error_type('fork_failed', $$)
		unless (defined $pid);

	if ( $pid ) {
		waitpid($pid,0)
			or return get_error_type('wait_on_pid', $pid);
	} else {
		$| = 1;
		close(STDOUT);
		open(STDOUT, ">$opts->{Target}") or exit($_exec_failed_open);
		close(STDERR);
		open(STDERR, ">>$_lynx_error_log");
		exec( $_lynx_location, $auth, "-error_file=$_lynx_error_log", 
			"-source", $opts->{URL});
		exit( $_exec_failed_code );
	}

	# File open (for writing) problem?
	return get_error_type ('openw_failed', $opts->{Target} )
		if ( $? >> 8 == $_exec_failed_open );

	# Exec failed
	return get_error_type( 'exec_failed', $_lynx_location )
		if ( $? >> 8 == $_exec_failed_code or $? & 127 or $? &128 );
	
	# Get the return code and unlink log file
	my $status = _returned_code( $_lynx_error_log );

	return $status;

}

sub _returned_code {
	my $lynx_error_log = shift;

	open( ERR_LOG, "$lynx_error_log" )
		or return get_error_type('log_open_error', $lynx_error_log);

	# get the last line - lynx sends 2 requests 
	# one without authorization data, one with

	my @lines = <ERR_LOG>;

	close( ERR_LOG );

	my $last_line = $lines[$#lines];

	# parse response code - lynx standard format of log file
	return get_error_type('log_parse_error', $_driver, $lynx_error_log, 
		"@lines")
		unless ( $last_line =~ /STATUS=(.*)$/ );

	my $response = $1;

	return get_error_type('log_parse_error', $_driver, $lynx_error_log,
		"@lines")
		unless ( $response =~ /HTTP\/\d\.\d\s+(\d+)/ );

	my $response_code = $1;

	# did manage to extract error code
	my $error_type = get_error_type( $response_code, $last_line );

	# unlink log file only after we were able to extract error code
	unlink $_lynx_error_log;

	return $error_type;
	}

1;
