# TUCOPY Tucows mirroring software
# Copyright (C) 2001 Tucows Inc.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. 
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details. 
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


#
# vi:set tabstop=4:
# $Id: Lock.pm,v 1.18 2001/05/10 20:29:11 bjovanovic Exp $
#
# Authors:
#       Leonid Igolnik <lim@tucows.com>
#       Bojan Jovanovic <bjovanovic@tucows.com>
#

package Tucows::Mirror::Client::Lock;

use IO::File;
use Tucows::Mirror::Log;

require Exporter;
@ISA = qw( Exporter );
@EXPORT_OK = qw( lock_file_name lock_file unlock_file touch_lock_file );

my $_default_timeout = 36000; #(10*60*60)

sub lock_file_name {
	my $filename = shift;
	return $filename . '.lock';
}

sub lock_file {
	my ($filename, $timeout) = @_;
	my $lock_filename = lock_file_name($filename);
	$timeout = $_default_timeout unless defined $timeout;

	if (-e $lock_filename) {
		my $age = (-M $lock_filename) * 24 * 60 * 60;
		if ($age > $timeout) {
			logit('error', 3, 'E7 Found stale lock %s, removing', $lock_filename);
			unlink($lock_filename) || do {
				logit('error', 3, 'E8 Unable to unlink stale lock file %s: %m',
						$lock_filename);
			};
		}
	}
	

	my $fh = IO::File->new($lock_filename,
							O_CREAT|O_EXCL|O_WRONLY) || do {
		logit('error', 4, 'E9 Unable to create lock file %s: %m', $lock_filename);
		return 0;
	};

	print $fh $$;
	$fh->close || do {
		logit('error', 4, 'E10 Unable to close lock file %s: %m', $lock_filename);
		unlink($lock_filename);
		return 0;
	};

	return 1;
}

sub touch_lock_file {
	my $filename = shift;
	my $now = time;

	utime($now, $now, lock_file_name($filename)) || 
		return 0;

	return 1;
	}

sub unlock_file {
	my $filename = shift;

	unlink(lock_file_name($filename)) || do {
		logit('error', 3, 'E11 Unable to remove lock file %s: %m',
				lock_file_name($filename));
		return 0;
	};

	return 1;
}

1;
