# TUCOPY Tucows mirroring software
# Copyright (C) 2001 Tucows Inc.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. 
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details. 
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


#
# vi:set tabstop=4:
# $Id: Gzip.pm,v 1.9 2001/04/16 22:03:48 bjovanovic Exp $
# 
# Authors:
#       Bojan Jovanovic <bjovanovic@tucows.com>
#       Leonid Igolnik <lim@tucows.com>
#

package Tucows::Mirror::Client::Uncompress::Gzip;

use Tucows::Mirror::Log;
use Tucows::Mirror::Config;

use strict;

my $_config = Tucows::Mirror::Config->new();
my $_gzip_location = $_config->compress_gzip_location;

# Error codes returned from the fork
my $_exec_failed_code = 201; #Should not colide with any of the gzip error codes
my $_exec_failed_open = 202; #Should not colide with any of the gzip error codes

# Null device
my $_null = '/dev/null';

sub uncompress_file {
	my $source = shift;
	my $dest = shift;

	if (! -x $_gzip_location) {
		logit('error', 1, 'E31 Cannot execute %s: %m', $_gzip_location);
		return 0;
	}

	if (! -r $source) {
		logit('error', 4, 'E32 Cannot read compressed file %s', $source);
		return 0;
	}

	my $pid = fork;

	if (not defined $pid) {
		logit('error', 2, 'E33 Fork failed Tucows::Mirror::Client::Uncompress::Gzip');
	}

	if ( $pid ) {
		waitpid($pid,0) or do {
			logit('error', 4, 'E34 waitpid failed in Tucows::Mirror::Client::Uncompress::Gzip');
			return 0;
		}
	} else {
		$| = 1;
		close(STDOUT);
		open(STDOUT, ">$dest") or exit($_exec_failed_open);
		close(STDERR);
		open(STDERR, ">$_null");
		exec( $_gzip_location, '-d', '-f', '-c', $source );
		exit( $_exec_failed_code );
	}

	if ( $? ) {

		logit('error', 2, 'E35 Exec of %s failed', $_gzip_location)
			if ( $? >> 8 == $_exec_failed_code);
		logit('error', 2, 'E36 Unable to open %s for writing', $dest)
    	    if ( $? >> 8 == $_exec_failed_open);

		return 0;
	} 

	return 1;
}

1;
