# TUCOPY Tucows mirroring software
# Copyright (C) 2001 Tucows Inc.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. 
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details. 
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


#
# vi:set tabstop=4:
# $Id: Zlib.pm,v 1.5 2001/04/12 18:50:52 bjovanovic Exp $
# 
# Authors:
#       Bojan Jovanovic <bjovanovic@tucows.com>
#       Leonid Igolnik <lim@tucows.com>
#

package Tucows::Mirror::Client::Uncompress::Zlib;

use Compress::Zlib;
use IO::File;
use Tucows::Mirror::Log;

use strict;

sub uncompress_file {
	my $source = shift;
	my $dest = shift;
	my $gz = gzopen($source, 'rb');
	my $out = IO::File->new();

	if (! -r $source) { return 0; }

	$out->open($dest,'w') ||
						do {
							logit('error', 2,
								'E37 Cannot open \'%s\' for writing: %m',
								$dest);
							return 0;
						};

	while (my $bytes = $gz->gzread($_)) {
		if ($bytes == -1) { 
			logit('error', 2, 'E38 Error reading compressed file %s: %m',
								$source);
			return 0;
		}
		print $out $_;
	}

	$gz->gzclose();
	$out->close() || 
				do {
					logit('error', 3, 'E39 Failed closing file %s', $dest);
					return 0;
				};
	return 1;
}

1;
