# TUCOPY Tucows mirroring software
# Copyright (C) 2001 Tucows Inc.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. 
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details. 
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


# 
# vi:set tabstop=4:
# $Id: Config.pm,v 1.17 2001/03/20 21:31:08 bjovanovic Exp $
#
# Configuration reading/parsing
#
# Authors:
#	Leonid Igolnik <lim@tucows.com>
# 	Bojan Jovanovic <bjovanovic@tucows.com>
#	Alex Nobert <anobert@tucows.com>
#
package Tucows::Mirror::Config;

use IO::File;
use Carp;

use vars qw( $AUTOLOAD );
use strict;

#default configuration file name
my $_default_config = 'tucopy.cfg';

#internal cache
my $_config_cache = {};

#making sure we are not autoloading DESTROY calls
sub DESTROY {
	return 1;
}

#Auto accessors for the config data
sub AUTOLOAD {
	my $self = shift;
	croak "$self is not an object ref in AUTOLOAD call $AUTOLOAD"
			unless ref($self);

	my $name = $AUTOLOAD;

	# strip fully-qualified portion
	$name =~ s/.*://;

	if (exists $self->{$name} ) {
		no strict 'refs';
#		*{$AUTOLOAD} = sub {
#				return $self->{$name};
#			};
		return $self->{$name};
	} else {
		warn "Attempting to retrive non-existant config parameter: '$name' from $self->{_file_name}";
		return undef;
	}
}

sub new {
	my $pkg = shift;

	#Config filename can come from params, ENV, or default config name
	my $file_name = shift || $ENV{MirrorConfigFile} || $_default_config;

	#check if the config info is already in the cache and return old object if
	#config is already cached
	return $_config_cache->{$file_name} if defined $_config_cache->{$file_name};

	#create new object
	my $config = {};
	bless $config, $pkg;

	#load info and store it in the cache, load returns object reference
	$_config_cache->{$file_name} = $config->load($file_name);
}

#Load configuration file
sub load {
	my $self = shift;
	my $file_name = shift;

	my $fh = IO::File->new($file_name, 'r') || do {
		warn "Unable to open config file '$file_name': $!\n";
		return undef;
	};

	$self->{_file_name} = $file_name;
	#$self->{_file_mtime} = ($fh->stat())[9];

	#read config from the file
	while (my $line = $fh->getline()) {
		chomp $line;

		#skip comments
		next if $line =~ /^#/;
			
		my ($name, $value) = split('=', $line, 2);
	
		#skip empty lines
		next unless (defined $name && $name =~ /\S/);

		$self->{$name} = $value;
	}	
	$fh->close();
	$self;
}

#return hash ref to the hash that contains all parametes
sub params {
	my $self = shift;
	my @keys = grep !/^_/, keys %{$self};
	my $result = {};
	foreach my $key (@keys) {
		$result->{$key} = $self->{$key};
	}
	return $result;
}

1;
