# TUCOPY Tucows mirroring software
# Copyright (C) 2001 Tucows Inc.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. 
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details. 
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


# 
# vi:set tabstop=4:
# $Id: Record.pm,v 1.27 2001/05/07 17:01:33 bjovanovic Exp $
#
# Authors:
#       Leonid Igolnik <lim@tucows.com>
#		Alex Nobert <anobert@tucows.com>
#       Bojan Jovanovic <bjovanovic@tucows.com>
# 

package Tucows::Mirror::Listing::Record;

use Tucows::Mirror::Config; 

use strict;


#Module defaults

my %_all_fields = (
	_file_name		=> {
				position => 1,
				save => 1,
			},
	_mtime			=> {
				position => 2,
				compare => 1,
				save => 1,
			},
	_file_size		=> {
				position => 3,
				compare => 1,
				save => 1,
			},
	_file_size_compressed	=> {
				position => 6,
				save => 1,
			},
	_is_compressed	=> {
				server_compare => 4,
				position => 4,
				save => 1,
			},
	_is_directory	=> {
				position => 5,
				save => 1,
			},
	_action			=> { },
);

my %_actions = (
	"update"			=> 'update',
	"delete"			=> 'delete',
);

my $_delim = '|';
my $_escaped_delim = '\|';
my $_compressed_ext;
my $_init = 0;

sub _init {
	$_compressed_ext = Tucows::Mirror::Config->new()->compressed_ext;
	$_init = 1;
}

sub new {
	my ($pkg, $params) = @_;
	my $record = {};

	_init() unless $_init;

	if ($params->{record}) {
		my @args = split($_escaped_delim, $params->{record});
		foreach my $field ( _get_sorted_element_list() ) {
		 	$record->{$field} = shift @args if $_all_fields{$field}{save};
		}
	} elsif ($params->{file_name} && $params->{root_dir} &&
				$params->{compressed_root_dir}) {
		my @file_stat = stat($params->{root_dir} . '/' . $params->{file_name});

		my @compressed_file_stat = stat ($params->{compressed_root_dir} . '/' .
											$params->{file_name} .
											$_compressed_ext)
			if $params->{server};
		$record->{_file_name} = $params->{file_name};
		$record->{_file_size} = $file_stat[7];
		$record->{_file_size_compressed} = $compressed_file_stat[7] || 0;
		$record->{_mtime} = $file_stat[9];
		$record->{_is_directory} = -d $params->{root_dir} . '/' .
										$params->{file_name} ? 1 : 0;
		$record->{_is_compressed} = $file_stat[9] && $compressed_file_stat[9] &&
									$file_stat[9] <= $compressed_file_stat[9] ?
									1 : 0;
		$record->{_action} = 0;
	} elsif ($params->{blank}) {
		foreach my $field (keys %_all_fields) {
			$record->{$field} = $params->{blank}
									if $_all_fields{$field}{save};
		}
	}
	bless $record, $pkg;
}

sub string {
	my $self = shift;
	my $string;
	foreach my $field ( _get_sorted_element_list() ) {
		if ($_all_fields{$field}{save}) {
			$string .= $self->{$field} . $_delim;
		}
	}
	return $string
}

sub _accessor {
	my $self = shift;
	my $param = shift;
	my $value = shift;
	if (defined $value) {
		$self->{$param} = $value;
	}
	return $self->{$param};
}

sub _get_sorted_element_list {
	return sort { ($_all_fields{$a}{position} || 0) <=>
						($_all_fields{$b}{position} || 0) }
					keys %_all_fields;
}

sub compare_record {
	my $self = shift;
	my $old_self = shift;
	my $server = shift;
	foreach my $field (keys %_all_fields) {
		if ( ($_all_fields{$field}{compare} ||
				($_all_fields{$field}{server_compare} && $server) ) &&
				( ! defined $self->{$field} || ! defined $old_self->{$field} ||
				$self->{$field} ne $old_self->{$field} ) ) {
			return 0;
		}
	}
	return 1;
}

sub file_name {
	my $self = shift;
	$self->_accessor('_file_name', shift);
}

sub file_size {
	my $self = shift;
	$self->_accessor('_file_size', shift);
}

sub file_size_compressed {
	my $self = shift;
	$self->_accessor('_file_size_compressed', shift);
}

sub mtime {
	my $self = shift;
	$self->_accessor('_mtime', shift);
}

sub is_compressed {
	my $self = shift;
	$self->_accessor('_is_compressed', shift);
}

sub is_directory {
	my $self = shift;
	$self->_accessor('_is_directory', shift);
}

sub action {
	my $self = shift;
	my $param = shift;
	$self->_accessor('_action', defined $param ? $_actions{$param} : undef);
}

1;
